@@ -35,7 +35,9 @@ void __connmanctl_dbus_print(DBusMessageIter *iter, const char *pre,
 	int arg_type;
 	dbus_bool_t b;
 	unsigned char c;
-	unsigned int i;
+	dbus_uint16_t u16;
+	dbus_uint32_t u;
+	dbus_int32_t i;
 	double d;
 
 	char *str;
@@ -97,7 +99,16 @@ void __connmanctl_dbus_print(DBusMessageIter *iter, const char *pre,
 			break;
 
 		case DBUS_TYPE_UINT16:
+			dbus_message_iter_get_basic(iter, &u16);
+			fprintf(stdout, "%u", u16);
+			break;
+
 		case DBUS_TYPE_UINT32:
+			dbus_message_iter_get_basic(iter, &u);
+			fprintf(stdout, "%d", u);
+			break;
+
+		case DBUS_TYPE_INT32:
 			dbus_message_iter_get_basic(iter, &i);
 			fprintf(stdout, "%d", i);
 			break;
@@ -216,16 +227,16 @@ static int append_variant(DBusMessageIter *iter, const char *property,
 	return 0;
 }
 
-int __connmanctl_dbus_method_call(DBusConnection *connection, const char *path,
-		const char *interface, const char *method,
-		connmanctl_dbus_method_return_func_t cb, void *user_data,
-		int arg1, ...)
+int __connmanctl_dbus_method_call(DBusConnection *connection,
+		const char *service, const char *path, const char *interface,
+		const char *method, connmanctl_dbus_method_return_func_t cb,
+		void *user_data, int arg1, ...)
 {
 	DBusMessage *message;
 	va_list args;
 
-	message = dbus_message_new_method_call("net.connman", path,
-			interface, method);
+	message = dbus_message_new_method_call(service, path, interface,
+			method);
 
 	if (message == NULL)
 		return -ENOMEM;
