@@ -50,7 +50,6 @@ int connman_inet_del_host_route(int index, const char *host);
 int connman_inet_add_network_route(int index, const char *host, const char *gateway,
 					const char *netmask);
 int connman_inet_del_network_route(int index, const char *host);
-int connman_inet_set_gateway_address(int index, const char *gateway);
 int connman_inet_clear_gateway_address(int index, const char *gateway);
 int connman_inet_set_gateway_interface(int index);
 int connman_inet_clear_gateway_interface(int index);
@@ -66,7 +65,6 @@ int connman_inet_add_ipv6_host_route(int index, const char *host,
 int connman_inet_del_ipv6_network_route(int index, const char *host,
 					unsigned char prefix_len);
 int connman_inet_del_ipv6_host_route(int index, const char *host);
-int connman_inet_set_ipv6_gateway_address(int index, const char *gateway);
 int connman_inet_clear_ipv6_gateway_address(int index, const char *gateway);
 int connman_inet_set_ipv6_gateway_interface(int index);
 int connman_inet_clear_ipv6_gateway_interface(int index);
