@@ -48,7 +48,7 @@
 #define SET_PROPERTY			"SetProperty"
 #define GET_DEVICES			"GetDevices"
 
-#define TIMEOUT 40000
+#define TIMEOUT 60000
 
 static DBusConnection *connection;
 
@@ -96,7 +96,7 @@ static int create_device(struct dundee_data *info)
 	DBG("%s", info->path);
 
 	ident = g_strdup(get_ident(info->path));
-	device = connman_device_create(ident, CONNMAN_DEVICE_TYPE_BLUETOOTH);
+	device = connman_device_create("dundee", CONNMAN_DEVICE_TYPE_BLUETOOTH);
 	if (device == NULL) {
 		err = -ENOMEM;
 		goto out;
@@ -181,8 +181,6 @@ static int create_network(struct dundee_data *info)
 	group = get_ident(info->path);
 	connman_network_set_group(network, group);
 
-	connman_network_set_available(network, TRUE);
-
 	err = connman_device_add_network(info->device, network);
 	if (err < 0) {
 		connman_network_unref(network);
@@ -632,12 +630,10 @@ static void add_device(const char *path, DBusMessageIter *properties)
 	if (info->active == TRUE)
 		set_connected(info);
 
+	return;
+
 out:
 	g_hash_table_remove(dundee_devices, path);
-
-	g_free(info->path);
-	g_free(info->name);
-	g_free(info);
 }
 
 static gboolean device_added(DBusConnection *conn, DBusMessage *message,
