@@ -97,6 +97,7 @@ struct wifi_data {
 	connman_bool_t disconnecting;
 	connman_bool_t tethering;
 	connman_bool_t bridged;
+	connman_bool_t interface_ready;
 	const char *bridge;
 	int index;
 	unsigned flags;
@@ -323,12 +324,16 @@ static int add_scan_param(gchar *hex_ssid, char *raw_ssid, int ssid_len,
 		 * used or are using multiple wifi cards, so in that case
 		 * you might have multiple service files for same AP.
 		 */
-		if (is_duplicate(scan_data->ssids, ssid, j) == TRUE)
+		if (is_duplicate(scan_data->ssids, ssid, j) == TRUE) {
+			if (hex_ssid != NULL)
+				g_free(ssid);
 			return 0;
+		}
 
 		scan_ssid = g_try_new(struct scan_ssid, 1);
 		if (scan_ssid == NULL) {
-			g_free(ssid);
+			if (hex_ssid != NULL)
+				g_free(ssid);
 			return -ENOMEM;
 		}
 
@@ -773,24 +778,9 @@ static void interface_autoscan_callback(int result,
 	}
 }
 
-static void interface_create_callback(int result,
-					GSupplicantInterface *interface,
-							void *user_data)
+static void finalize_interface_creation(struct wifi_data *wifi)
 {
-	struct wifi_data *wifi = user_data;
-
-	DBG("result %d ifname %s, wifi %p", result,
-				g_supplicant_interface_get_ifname(interface),
-				wifi);
-
-	if (result < 0 || wifi == NULL)
-		return;
-
-	wifi->interface = interface;
-	g_supplicant_interface_set_data(interface, wifi);
-
-	if (g_supplicant_interface_get_ready(interface) == FALSE)
-		return;
+	GSupplicantInterface *interface = wifi->interface;
 
 	DBG("interface is ready wifi %p tethering %d", wifi, wifi->tethering);
 
@@ -812,6 +802,28 @@ static void interface_create_callback(int result,
 	}
 }
 
+static void interface_create_callback(int result,
+					GSupplicantInterface *interface,
+							void *user_data)
+{
+	struct wifi_data *wifi = user_data;
+
+	DBG("result %d ifname %s, wifi %p", result,
+				g_supplicant_interface_get_ifname(interface),
+				wifi);
+
+	if (result < 0 || wifi == NULL)
+		return;
+
+	wifi->interface = interface;
+	g_supplicant_interface_set_data(interface, wifi);
+
+	if (g_supplicant_interface_get_ready(interface) == TRUE) {
+		wifi->interface_ready = TRUE;
+		finalize_interface_creation(wifi);
+	}
+}
+
 static int wifi_enable(struct connman_device *device)
 {
 	struct wifi_data *wifi = connman_device_get_data(device);
@@ -1562,10 +1574,18 @@ static void interface_state(GSupplicantInterface *interface)
 	if (wifi == NULL)
 		return;
 
-	network = wifi->network;
 	device = wifi->device;
+	if (device == NULL)
+		return;
 
-	if (device == NULL || network == NULL)
+	if (g_supplicant_interface_get_ready(interface) == TRUE &&
+					wifi->interface_ready == FALSE) {
+		wifi->interface_ready = TRUE;
+		finalize_interface_creation(wifi);
+	}
+
+	network = wifi->network;
+	if (network == NULL)
 		return;
 
 	switch (state) {
@@ -1967,6 +1987,7 @@ static void ap_create_callback(int result,
 		connman_technology_tethering_notify(info->technology, FALSE);
 
 		g_free(info->ifname);
+		g_free(info->ssid);
 		g_free(info);
 		return;
 	}
@@ -1994,6 +2015,7 @@ static void sta_remove_callback(int result,
 		info->wifi->tethering = TRUE;
 
 		g_free(info->ifname);
+		g_free(info->ssid);
 		g_free(info);
 		return;
 	}
@@ -2069,6 +2091,7 @@ static int tech_set_tethering(struct connman_technology *technology,
 		}
 		info->ifname = g_strdup(ifname);
 		if (info->ifname == NULL) {
+			g_free(info->ssid);
 			g_free(info);
 			continue;
 		}
